/* ------------------------------------------------------------------------------------------------------------------------------
    adav801 main functions.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   ----------------------------------------------------------------------------------------------------------------------------- */

#include <linux/spi/spi.h>
#include <mach/gpio.h>// pin numbers

#define SPI_1_SPEED 4000000
#define ADAV801_CS_PIN	AT91_PIN_PC7

/* ----------------------------------------------------------------------------------------------------------------------------- */
/* MODULE INIT/EXIT                                                                                                              */
/* ----------------------------------------------------------------------------------------------------------------------------- */

static struct spi_board_info spi_adav801_device = 
{
	.modalias		= "adav801",
	.chip_select	= 0,
	.max_speed_hz	= SPI_1_SPEED, 
	.controller_data	= (void *) ADAV801_CS_PIN,
	.bus_num		= 1,
	.mode			= SPI_MODE_0,
};

static void __init write_adav801(struct spi_device* spi_codec, u8 addr, u8 val) 
{
    u8 reg[2];
    reg[0] = addr;
    reg[1] = val;
    spi_write( spi_codec, reg, sizeof(reg) );
}

int __init snd_master_clock_init(void)
{
	static struct spi_device* spi_codec;
	struct spi_master* spi = spi_busnum_to_master( spi_adav801_device.bus_num );
	if( spi == NULL )
	{
		printk(KERN_ERR "unable to get spi master %d\n", spi_adav801_device.bus_num);
		return -1;
	}

	spi_codec = spi_new_device( spi, &spi_adav801_device );
	if( spi_codec == NULL )
	{
		printk(KERN_ERR "unable to create spi device for adav801\n");
		return -1;
	}

	// SYSCLK1 = 8192MHz = 256 * 32000
	// SYSCLK2 = 12.288MHz = 256 * 48000
	// SYSCLK3 = 24.576MHz = 512 * 48000
	//write_adav801(0x00, 0x00);
	write_adav801(spi_codec, 0x74<<1, 0x00);//512*48=SYSCLCK3, MCLKO=27MHz
	write_adav801(spi_codec, 0x75<<1, 0x08);//32*256=SYSCLK1, 48*256=SYSCLK2
	//write_adav801(0x76, 0x4B);
	//write_adav801(0x77, 0x18);//source for ICLK = PLLINT2 = 12.288MHz
	write_adav801(spi_codec, 0x78<<1, 0x00);//PLL1CLK=PLL2CLK=XIN

	spi_unregister_device(spi_codec);
	return 0;
}

void __exit snd_master_clock_exit(void)
{
}
